﻿using System;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using Microsoft.WindowsAzure.StorageClient;
using Microsoft.WindowsAzure;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using System.Collections.Generic;
using System.Configuration;
using System.Web.UI.WebControls;



namespace MSSAzureArchive.Layouts.MSSAzureArchive
{
    public partial class AzureStorage : LayoutsPageBase
    {
        private CloudBlobClient cloudBlobClient;

        private SPWeb webSite;
        SPSite webSiteCollection;
        private string filePath;
        private SPFile fileToArchive;

        private string accountName;
        private string accountSharedKey;
        private string defaultContainerName;

        private CloudBlob cloudBlob;
        private CloudBlobContainer blobContainer;

        private Uri blobUri;
        private Uri queueUri;
        private Uri tableUri;

        protected void Page_Load(object sender, EventArgs e)
        {

            this.InitializeCloudStorage();
            if (!IsPostBack)
            {
                this.GetContainers();
            }
        }
        protected void SaveContainer_Click(object o, EventArgs e)
        {
            try
            {
                CloudBlobContainer newContainer = cloudBlobClient.GetContainerReference(this.newContainerName.Text);
                newContainer.CreateIfNotExist();
                var perms = newContainer.GetPermissions();
                perms.PublicAccess = BlobContainerPublicAccessType.Container;
                newContainer.SetPermissions(perms);
            }
            catch (Exception ex)
            {
                //display the error on the screen.
                errMessage.Text = ex.Message;
            }

            GetContainers();
            this.azureContainers.Items.FindByValue(newContainerName.Text).Selected = true;
            this.azureContainers.Visible = true;
            this.newContainerName.Visible = false;
            this.saveContainer.Visible = false;
        }
        protected void CreateContainer_Click(object o, EventArgs e)
        {
            this.azureContainers.Visible = false;
            this.newContainerName.Visible = true;
            this.saveContainer.Visible = true;

        }
        protected void Archive_Click(object o, EventArgs e)
        {
            try
            {
                webSite = SPContext.Current.Web;
                filePath = webSite.Url.ToString() + Request.QueryString["ItemUrl"].ToString();
                fileToArchive = webSite.GetFile(filePath);
                string sArchived = fileToArchive.Item["IsArchived"].ToString(); 

                bool isArchived = Convert.ToBoolean(sArchived);

                if (isArchived)//Item is flagged as archived
                {
                    errMessage.Text = "This document has already been archived!";
                }
                else//Archive to Azure
                {
                    string newGuid = Guid.NewGuid().ToString();
                    string uniqueBlobName = string.Format(newGuid + "_" + fileToArchive.Name.ToString());

                    blobContainer = cloudBlobClient.GetContainerReference(this.azureContainers.SelectedValue);
                    blobContainer.CreateIfNotExist();
                    cloudBlob = blobContainer.GetBlockBlobReference(uniqueBlobName);
                    cloudBlob.UploadByteArray(fileToArchive.OpenBinary());

                    SPDocumentLibrary docLib = fileToArchive.DocumentLibrary;

                    Hashtable docProperties = new Hashtable();
                    docProperties["IsArchived"] = true;
                    string docLibRelPath = docLib.RootFolder.ServerRelativeUrl;
                    string docLibPath = string.Empty;


                    webSiteCollection = SPContext.Current.Site;
                    docLibPath = webSiteCollection.MakeFullUrl(docLibRelPath);

                    string documentPath = docLibPath + "/" + fileToArchive.Name.ToString() + ".aspx";
                    string azureURL = cloudBlob.Uri.ToString();
                    string azureStub = this.BuildLinkToItem(azureURL).ToString();

                    SPFile newFile = webSite.Files.Add(documentPath, UTF8Encoding.UTF8.GetBytes(azureStub), docProperties, true);

                    SPListItem item = newFile.Item;
                    item["Content Type"] = "Link to a Document";
                    SPFieldUrlValue itemUrl = new SPFieldUrlValue();
                    itemUrl.Description = fileToArchive.Name;
                    itemUrl.Url = azureURL;
                    item["URL"] = itemUrl;
                    item["IsArchived"] = true;
                    item.Update();
                    fileToArchive.Delete();
                }
            }

            catch (Exception ex)
            {
                errMessage.Text = "Error: " + ex.Message;
            }
        }
        private void InitializeCloudStorage()
        {
            try
            {
                accountName = ConfigurationSettings.AppSettings["AccountName"];
                accountSharedKey = ConfigurationSettings.AppSettings["AccountSharedKey"];
                blobUri = new Uri(ConfigurationSettings.AppSettings["BlobStorageEndPoint"]);
                queueUri = new Uri(ConfigurationSettings.AppSettings["QueueStorageEndPoint"]);
                tableUri = new Uri(ConfigurationSettings.AppSettings["TableStorageEndPoint"]);
                defaultContainerName = ConfigurationSettings.AppSettings["DefaultContainerName"].ToString();

                this.fileName.Text = Request.QueryString["ItemUrl"].ToString();

                CloudStorageAccount azAccount = new CloudStorageAccount
                (
                    new StorageCredentialsAccountAndKey(accountName, accountSharedKey),
                    blobUri,
                    queueUri,
                    tableUri
                );

                cloudBlobClient = azAccount.CreateCloudBlobClient();
            }
            catch (Exception ex)
            {
                errMessage.Text = ex.Message;
            }
        }
        private StringBuilder BuildLinkToItem(string azureURL)
        {
            StringBuilder azureLinkItem = new StringBuilder();
            string linkItemText = "<%@ Assembly Name='Microsoft.SharePoint, Version=12.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c' %> " +
                        "<%@ Register TagPrefix='SharePoint' Namespace='Microsoft.SharePoint.WebControls' Assembly='Microsoft.SharePoint' %>" +
                        "<%@ Import Namespace='System.IO' %>" +
                        "<%@ Import Namespace='Microsoft.SharePoint' %>" +
                        "<%@ Import Namespace='Microsoft.SharePoint.Utilities' %>" +
                        "<%@ Import Namespace='Microsoft.SharePoint.WebControls' %>" +
                        "<html xmlns:mso=\"urn:schemas-microsoft-com:office:office\" xmlns:msdt=\"uuid:C2F41010-65B3-11d1-A29F-00AA00C14882\">" +
                        "<Head> <META Name='progid' Content='SharePoint.Link'>" +
                        "<!--[if gte mso 9]><xml>" +
                        "<mso:CustomDocumentProperties>" +
                        "<mso:URL msdt:dt=\"string\">http://bing.com,http://bing.com</mso:URL>" +
                        "<mso:ContentType msdt:dt=\"string\">Link to a Document</mso:ContentType>" +
                        "</mso:CustomDocumentProperties>" +
                        "</xml><![endif]-->" +
                        "</head>" +
                        "    <body>" +
                        "        <form id='Form1' runat='server'>" +
                        "            <SharePoint:UrlRedirector id='Redirector1' runat='server' />" +
                        "        </form>" +
                        "    </body>" +
                        "</html>";
            azureLinkItem.Append(linkItemText);
            azureLinkItem.Replace("http://bing.com", azureURL);

            return azureLinkItem;
        }
        private void GetContainers()
        {
            IEnumerable<CloudBlobContainer> blobContainers = cloudBlobClient.ListContainers();

            this.azureContainers.DataSource = blobContainers;
            azureContainers.DataTextField = "Name";
            this.azureContainers.DataBind();
            if (azureContainers.Items.Count < 1)
            {
                ListItem defaultContainer = new ListItem(defaultContainerName);
                defaultContainer.Selected = true;
                azureContainers.Items.Add(defaultContainer);
            }
        }
    }
}
